module logicanalyser1 (clockin, reset, reading, address, startcapture, nextaddress, ce, we, oe);

input clockin;
input reset;
input reading;

output [16:0] address;
reg [16:0] address;
reg [16:0] startaddress;

input startcapture;
reg startcaptureinit;
reg capturedone;

input nextaddress;
reg nextaddresswait;
reg readinginit;

output ce;
wire ce;
output we;
reg we;
output oe;
reg oe;

always @ (posedge clockin) begin
	if (!reset) begin
		address <= 17'b0;
		nextaddresswait <= 1'b0;
		readinginit <= 1'b1;
		startcaptureinit <= 1'b1;
		capturedone <= 1'b0;
		we <= 1'b1;
		oe <= 1'b1;
	end
	else begin
		// Start writing and wait for a trigger as long as the capture isn't finished
		if (!reading && !capturedone) begin
			we <= 1'b0;
			oe <= 1'b1;
			readinginit <= 1'b1;
			address <= address + 17'b1;
			
			// Triggered, save the starting address
			if (startcapture && startcaptureinit) begin
				startaddress <= address - 17'd50;
				startcaptureinit <= 1'b0;
			end
			
			// Current address is now the start address, stop the capture
			if (startcapture && (startaddress == address)) begin
			  capturedone <= 1'b1;
			end
		end
		
		// Start reading
		else if (reading) begin
			oe <= 1'b0;
			we <= 1'b1;
		  
			// Initialise the starting read address as the start of the capture
			if (readinginit == 1) begin
				readinginit <= 1'b0;
				address <= startaddress;
			end
		  
			// Increment address by 1 when we receive a low and then high clock
			else if (nextaddress == 1 && nextaddresswait == 0) begin
				nextaddresswait <= 1'b1;
				address <= address + 17'b1;
			end
			else if (nextaddress == 0) begin
				nextaddresswait <= 1'b0;
			end
		end
	end
end

assign ce = (reset) ? clockin : 1'b1;

endmodule


// not tested but may work
/*module logicanalyser1 (clockin,
reset,
writing,
readpulse,
address,
//data, 
ce);

input clockin;
input reset;
input writing;
input readpulse;

output [16:0] address;
reg [16:0] address;
//inout [7:0] data;
//reg [7:0] data;
reg readok;
output ce;
wire ce;

reg [16:0] oldaddress;
reg writeok;

reg initread;

always @ (posedge clockin) begin
	if (!reset) begin
		address <= 17'b0;
		//data <= 8'bz;
		readok <= 1'b0;
		writeok <= 1'b0;
		initread <= 1'b0;
	end
	else begin
		address <= address + 17'b1;
		//data <= data + 8'b1;
		if (writing && writeok == 0) begin
			oldaddress <= address - 17'd50;
			writeok <= 1'b1;
		end
		else if (!writing) begin 
			//data <= 8'bz;
			if (initread == 0) begin
				address <= oldaddress;
				initread <= 1'b1;
			end			
			else if (readpulse == 1 && readok == 0) begin
				readok <= 1'b1;
				address <= address + 17'b1;
			end
			else if (readpulse == 0) begin
				readok <= 1'b0;
			end
		end
	end
end

assign ce = (reset) ? clockin : 1'b1;

endmodule

*/


/*module logicanalyser1 (clockin,
reset,
writing,
readpulse,
address,
data, 
ce);

input clockin;
input reset;
input writing;
input readpulse;

output [16:0] address;
reg [16:0] address;
inout [7:0] data;
reg [7:0] data;
reg readok;
output ce;
wire ce;

always @ (posedge clockin) begin
	if (!reset) begin
		address <= 17'b0;
		data <= 8'bz;
		readok <= 1'b0;
	end
	else begin
		if (writing) begin
			address <= address + 17'b1;
			data <= data + 8'b1;
		end
		else begin 
			data <= 8'bz;
			if (readpulse == 1 && readok == 0) begin
				readok <= 1'b1;
				address <= address + 17'b1;
			end
			if (readpulse == 0) begin
				readok <= 1'b0;
			end
		end
	end
end

assign ce = (reset) ? clockin : 1'b1;

endmodule

*/

